local tomQuest2 = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")

-----------------------
--- Library Loading ---
-----------------------
local LCo = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Config")
local LSM = LibStub("LibSharedMedia-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")

--------------------------------------------
--- Copy global functions to local scope ---
--------------------------------------------
local CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName = CreateMacro, EditMacro, PickupMacro, GetMacroIndexByName
local pairs, type, sub, tonumber, format, unpack, IsAddOnLoaded = pairs, type, string.sub, tonumber, string.format, unpack, IsAddOnLoaded

-----------------
--- Variables ---
-----------------
local outlines = {[""] = LCo["None"], ["OUTLINE"] = LCo["Outline"], ["THICKOUTLINE"] = LCo["Thick outline"]}
local raidModeValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
}
local raidModeTrackerValues = {
	["enabled"] = LCo["Enabled"],
	["inRaid"] = LCo["Disabled once entering a raid"],
	["inRaidInstance"] = LCo["Disabled once entering a raid instance (will work in pvp and outdoor raids)"],
	["hiddenInRaid"] = LCo["Hidden once entering a raid"],
	["hiddenInRaidInstance"] = LCo["Hidden once entering a raid instance (will work in pvp and outdoor raids)"],
}
local questLevelTag = {
	["questLevel"] = LCo["Quest level"],
	["weightedQuestLevel"] = LCo["Weighted quest level"],
	["Hide"] = LCo["Hide"],
}
local zSorting = {
	["levelA"] = LCo["Sort zones based on level (ascending)"],
	["levelD"] = LCo["Sort zones based on level (descending)"],
	["nameA"] = LCo["Sort zones based on zone name (ascending)"],
	["nameD"] = LCo["Sort zones based on zone name (descending)"],
}
local zwlPosition = {
	[0] = LCo["Zones without level first"],
	[100] = LCo["Zones without level last"]
}
local qSorting = {
	["weightedA"] = LCo["Sort quests based on weighted level (ascending)"],
	["weightedD"] = LCo["Sort quests based on weighted level (descending)"],
	["levelA"] = LCo["Sort quests based on quest level (ascending)"],
	["levelD"] = LCo["Sort quests based on quest level (descending)"],
	["nameA"] = LCo["Sort quests based on name (ascending)"],
	["nameD"] = LCo["Sort quests based on name (descending)"],
}
local achievementPosition = {[1] = LCo["Above"], [2] = LCo["Below"]}
local achievementInteractivityList = {
	["chatLink"] = LCo["Insert achievement link in chat"],
	["tooltip"] = LCo["Display of achievement tooltip"],
	["removeTrackedAchievement"] = LCo["Remove achievement from tracker"],
	["displayAchievement"] = LCo["Diplay achievement frame"],
	["nothing"] = LCo["Do nothing"],
}
local ldbInteractivityList = {
	["openOptions"] = LCo["Open tomQuest2 option frame"],
	["toggleQuestTracker"] = LCo["Toggle quests tracking frame"],
	["toggleAchievementsTracker"] = LCo["Toggle achievements tracking frame"],
	["toggleAnnouncementModule"] = LCo["toggle chat announcement module"],
	["toggleQuestsLog"] = LCo["Toggle Blizzard quests log"],
	["toggleSyncPartyModule"] = LCo["Toggle party synchronization module"],
	["toggleQuestsTrackerModule"] = LCo["Toggle quests tracking module"],
	["toggleAchievementsTrackerModule"] = LCo["Toggle achievements tracking module"],
	["lockQuestsTracker"] = LCo["Lock - unlock quests tracker"],
	["lockAchievementsTracker"] = LCo["Lock - unlock achievements tracker"],
	["nothing"] = LCo["Do nothing"],
}
local anchorPointLOV = {
	["TOPLEFT"] = LCo["TOPLEFT"],
	["TOPRIGHT"] = LCo["TOPRIGHT"],
	["BOTTOMLEFT"] = LCo["BOTTOMLEFT"],
	["BOTTOMRIGHT"] = LCo["BOTTOMRIGHT"],
}
local automaticAcceptChoices = {[1] = LCo["All quests"], [2] = LCo["Only daily quests"], [3] = LCo["All but daily quests"], [4] = LCo["Never"]}
local attachSelect = {[1] = LCo["Right"], [2] = LCo["Left"]}
local hideObjectives = {[1] = LCo["When you've completed the objective"], [2] = LCo["When everyone in the party has completed the objective"], [3] = LCo["Never"]}
local questsAutoTrack = {[1] = LCo["Always"], [2] = LCo["Below max level"], [3] = LCo["Never"]}
local hideQuests = {[1] = LCo["When you've completed the quest"], [2] = LCo["When everyone in the party has completed the quest"], [3] = LCo["Never"]}
local overridePointY = {TOPLEFT = UIParent:GetHeight(), BOTTOMLEFT = 0, TOPRIGHT = UIParent:GetHeight(), BOTTOMRIGHT = 0}
local overridePointX = {TOPLEFT = 0, BOTTOMLEFT = 0, TOPRIGHT = UIParent:GetWidth(), BOTTOMRIGHT = UIParent:GetWidth()}

----------------------
--- Hide functions ---
----------------------
local function hideCustomColorMode()
	if tomQuest2.db.profile.customColorMode then
		return true
	else
		return false
	end
end
local function hideAchievementInteraction()
	if tomQuest2.db.profile.achievementsTracker.disableInteraction then
		return true
	else
		return false
	end
end
local function hideQuestsTrackerInteraction()
	if tomQuest2.db.profile.questsTracker.disableInteraction then
		return true
	else
		return false
	end
end

-----------------------
--- Tools functions ---
-----------------------
local function getColorFromCompletionHEX(percent)
	if percent <= 0 then
		return format("%02x%02x%02x", 255*tomQuest2.db.profile.rj0, 255*tomQuest2.db.profile.gj0, 255*tomQuest2.db.profile.bj0)
	elseif percent >= 1 then
		return format("%02x%02x%02x", 255*tomQuest2.db.profile.rj3, 255*tomQuest2.db.profile.gj3, 255*tomQuest2.db.profile.bj3)
	elseif percent <= 0.5 then
		return format("%02x%02x%02x", 255*normalizeColorComponent(tomQuest2.db.profile.ri1*percent + tomQuest2.db.profile.rj1), 255*normalizeColorComponent(tomQuest2.db.profile.gi1*percent + tomQuest2.db.profile.gj1), 255*normalizeColorComponent(tomQuest2.db.profile.bi1*percent + tomQuest2.db.profile.bj1))
	else
		return format("%02x%02x%02x", 255*normalizeColorComponent(tomQuest2.db.profile.ri2*percent + tomQuest2.db.profile.rj2), 255*normalizeColorComponent(tomQuest2.db.profile.gi2*percent + tomQuest2.db.profile.gj2), 255*normalizeColorComponent(tomQuest2.db.profile.bi2*percent + tomQuest2.db.profile.bj2))
	end
end
local function computeQuestsStatus()
	for qid in pairs(tomQuest2.quests) do
		tomQuest2.updateQuestCells(qid)
	end
end
local function computeQuestsObjectivesStatus()
	for qid, qoData in pairs (tomQuest2.questsObjectives) do
		for oid in pairs (qoData) do
			tomQuest2.updateQuestObjectiveCells(qid, oid)
		end
	end
end
local function normalizeColorComponent(component)
	if component > 1 then return 1 elseif component < 0 then return 0 else return component end
end
local function computeFirstFunction(r0, g0, b0, rh, gh, bh)
	tomQuest2.db.profile.rj1 = r0
	tomQuest2.db.profile.gj1 = g0
	tomQuest2.db.profile.bj1 = b0
	tomQuest2.db.profile.ri1 = 2 * (rh - r0)
	tomQuest2.db.profile.gi1 = 2 * (gh - g0)
	tomQuest2.db.profile.bi1 = 2 * (bh - b0)
end
local function computeSecondFunction(rh, gh, bh, r1, g1, b1)
	tomQuest2.db.profile.rj2 = 2 * rh - r1
	tomQuest2.db.profile.gj2 = 2 * gh - g1
	tomQuest2.db.profile.bj2 = 2 * bh - b1
	tomQuest2.db.profile.ri2 = 2 * (r1 - rh)
	tomQuest2.db.profile.gi2 = 2 * (g1 - gh)
	tomQuest2.db.profile.bi2 = 2 * (b1 - bh)
end
local function globalFont()
	_G["cATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.categoryFont), tomQuest2.db.profile.achievementsTracker.categoryFontSize, tomQuest2.db.profile.achievementsTracker.categoryFontOutline)
	_G["scATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.subCategoryFont), tomQuest2.db.profile.achievementsTracker.subCategoryFontSize, tomQuest2.db.profile.achievementsTracker.subCategoryFontOutline)
	_G["aATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementFont), tomQuest2.db.profile.achievementsTracker.achievementFontSize, tomQuest2.db.profile.achievementsTracker.achievementFontOutline)
	_G["acATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementCriterionFont), tomQuest2.db.profile.achievementsTracker.achievementCriterionFontSize, tomQuest2.db.profile.achievementsTracker.achievementCriterionFontOutline)
	_G["sbATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.statusBarFont), tomQuest2.db.profile.achievementsTracker.statusBarFontSize, tomQuest2.db.profile.achievementsTracker.statusBarFontOutline)
	_G["cQToFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.categoryFont), tomQuest2.db.profile.questsTooltip.categoryFontSize, tomQuest2.db.profile.questsTooltip.categoryFontOutline)
	_G["qQToFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.questFont), tomQuest2.db.profile.questsTooltip.questFontSize, tomQuest2.db.profile.questsTooltip.questFontOutline)
	_G["qoQToFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.questObjectiveFont), tomQuest2.db.profile.questsTooltip.questObjectiveFontSize, tomQuest2.db.profile.questsTooltip.questObjectiveFontOutline)
	_G["cQTrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.categoryFont), tomQuest2.db.profile.questsTracker.categoryFontSize, tomQuest2.db.profile.questsTracker.categoryFontOutline)
	_G["qQTrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.questFont), tomQuest2.db.profile.questsTracker.questFontSize, tomQuest2.db.profile.questsTracker.questFontOutline)
	_G["qoQTrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.questObjectiveFont), tomQuest2.db.profile.questsTracker.questObjectiveFontSize, tomQuest2.db.profile.questsTracker.questObjectiveFontOutline)
	tomQuest2.setAchievementsTrackerLayout()
	tomQuest2.setQuestsTrackerLayout()
	tomQuest2.refreshQuestInformation(qid)
end
local function specialItemMacro(info, value)
	local index = GetMacroIndexByName("tomQuest2_SI")
	if index == 0 then
		index = CreateMacro("tomQuest2_SI", 1, "", 0)
		PickupMacro(index)
	else
		EditMacro(index, "tomQuest2_SI", 1, "")
	end
end
local function setDefaultColor()
	tomQuest2.db.profile.rj0, tomQuest2.db.profile.gj0, tomQuest2.db.profile.bj0 = 1, 0, 0
	tomQuest2.db.profile.ri1, tomQuest2.db.profile.rj1, tomQuest2.db.profile.gi1, tomQuest2.db.profile.gj1, tomQuest2.db.profile.bi1, tomQuest2.db.profile.bj1 = 0, 1, 2, 0, 0 ,0
	tomQuest2.db.profile.ri2, tomQuest2.db.profile.rj2, tomQuest2.db.profile.gi2, tomQuest2.db.profile.gj2, tomQuest2.db.profile.bi2, tomQuest2.db.profile.bj2 = -2, 2, 0, 1, 0, 0
	tomQuest2.db.profile.rj3, tomQuest2.db.profile.gj3, tomQuest2.db.profile.bj3 = 0, 1, 0
	computeQuestsObjectivesStatus()
end
local function setTrackerpoint(self)
		local containerType, width, height = self.containerType, UIParent:GetWidth(), UIParent:GetHeight()
		local hhalf, xOffset, vhalf, yOffset, x, y
		if tomQuest2.db.profile[containerType].overrideAnchor then
			local overridePoint = tomQuest2.db.profile[containerType].overridePoint
			x, y = overridePointX[overridePoint], overridePointY[overridePoint]
		else
			x, y = self:GetCenter()
		end
		if x > width/2 then
			hhalf = "RIGHT"
			xOffset = -(width - self:GetRight())
		else
			hhalf = "LEFT"
			xOffset = self:GetLeft()
		end
		if y > height/2 then
			vhalf = "TOP"
			yOffset = -(height - self:GetTop())
		else
			vhalf = "BOTTOM"
			yOffset = self:GetBottom()
		end
		local point = vhalf..hhalf
		tomQuest2.db.profile[containerType].point, tomQuest2.db.profile[containerType].relativePoint, tomQuest2.db.profile[containerType].xOffset, tomQuest2.db.profile[containerType].yOffset = point, point, xOffset, yOffset
		self:ClearAllPoints()
		self:SetPoint(tomQuest2.db.profile[containerType].point, UIParent, tomQuest2.db.profile[containerType].relativePoint, xOffset, yOffset)
end

-----------------------
--- Getter & Setter ---
-----------------------
local function optGetter(info)
	if info[#info-1] == "modules" then
		return tomQuest2.db.profile.modules[info[#info]]
	elseif info[#info-1] == "raidMode" then
		return tomQuest2.db.profile.raidModules[info[#info]]
	elseif info[#info-1] == "questStatusColor" then
		return tonumber(sub(tomQuest2.db.profile[info[#info]], 1, 2), 16)/255, tonumber(sub(tomQuest2.db.profile[info[#info]], 3, 4), 16)/255, tonumber(sub(tomQuest2.db.profile[info[#info]], 5, 6), 16)/255
	elseif info[#info] == "zeroColor" then
		return normalizeColorComponent(tomQuest2.db.profile.rj0) , normalizeColorComponent(tomQuest2.db.profile.gj0), normalizeColorComponent(tomQuest2.db.profile.bj0)
	elseif info[#info] == "halfColor" then
		return normalizeColorComponent(0.5*tomQuest2.db.profile.ri1 + tomQuest2.db.profile.rj1), normalizeColorComponent(0.5*tomQuest2.db.profile.gi1 + tomQuest2.db.profile.gj1), normalizeColorComponent(0.5*tomQuest2.db.profile.bi1 + tomQuest2.db.profile.bj1)
	elseif info[#info] == "fullColor" then
		return normalizeColorComponent(tomQuest2.db.profile.rj3), normalizeColorComponent(tomQuest2.db.profile.gj3), normalizeColorComponent(tomQuest2.db.profile.bj3)
	elseif info[#info] == "questTitleColor" then
		return tonumber(sub(tomQuest2.db.profile[info[#info]], 1, 2), 16) / 255,
		tonumber(sub(tomQuest2.db.profile[info[#info]], 3, 4), 16) / 255, 
		tonumber(sub(tomQuest2.db.profile[info[#info]], 5, 6), 16) / 255
	else
		return tomQuest2.db.profile[info[#info]]
	end
end
local function optSetter(info, arg1, arg2, arg3, arg4)
	if info[#info-1] == "modules" then
		tomQuest2.db.profile.modules[info[#info]] = arg1
		if arg1 then
			tomQuest2:enableModule(info[#info])
		else
			tomQuest2:disableModule(info[#info])
		end
	elseif info[#info-1] == "raidMode" then
		local oldState = tomQuest2.db.profile.raidModules.enabled
		tomQuest2.db.profile.raidModules[info[#info]] = arg1
		if tomQuest2.db.profile.raidModules.enabled then
			tomQuest2:RAID_ROSTER_UPDATE(nil, true)
		elseif oldState then
			for k, state in pairs(tomQuest2.db.profile.modules) do
				if state and not tomQuest2.moduleState[k] then
					tomQuest2:enableModule(k)
					tomQuest2:Print(format(LCo["%s module is now enabled"], k))
				end
				if tomQuest2.db.profile.raidModules[k] == "hiddenInRaid" or tomQuest2.db.profile.raidModules[k] == "hiddenInRaidInstance" then
					if k == "questsTracker" then
						tomQuest2:tQ2toggleQuestTracker(1)
					elseif k == "achievementsTracker" then
						tomQuest2:tQ2toggleAchievementsTracker(1)
					end
				end
			end
		end
	elseif info[#info-1] == "questStatusColor" then
		tomQuest2.db.profile[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
		tomQuest2:updateQuestCounter()
		computeQuestsStatus()
	else
		tomQuest2.db.profile[info[#info]] = arg1
		if info[#info] == "hideDefaultBlizTracker" then
			tomQuest2:hideShowDefaultBlizTracker()
		elseif info[#info] == "partyHeader" then
			local partyCategory = tomQuest2.PARTYQUESTSZONE
			if arg1 then
				for qid in pairs(tomQuest2.quests) do
					if not tomQuest2.quests[qid].playerStatus then
						local category = tomQuest2.quests[qid].category
						tomQuest2.questsByCategory[category][qid] = nil
						if not next(tomQuest2.questsByCategory[category]) then
							tomQuest2.removeQuestCategoryCells(category)
							tomQuest2.questsCategories[category], tomQuest2.questsByCategory[category], tomQuest2.questsByCategory[category] = nil, nil, nil
						end
						if not tomQuest2.questsCategories[partyCategory] then
							tomQuest2:addQuestCategory(partyCategory)
						end
						tomQuest2.questsByCategory[partyCategory][qid] = true
					end
				end
				tomQuest2:questsSort()
				tomQuest2.setQuestsTrackerLayout()
			elseif tomQuest2.questsByCategory[partyCategory] then
				for qid in pairs(tomQuest2.questsByCategory[partyCategory]) do
					local zone = tomQuest2.quests[qid].category
					if not tomQuest2.questsCategories[zone] then
						tomQuest2:addQuestCategory(zone)
					end
					tomQuest2.questsByCategory[zone][qid] = true
				end
				tomQuest2.removeQuestCategoryCells(partyCategory)
				tomQuest2.questsCategories[partyCategory], tomQuest2.questsByCategory[partyCategory] = nil, nil
				tomQuest2:questsSort()
				tomQuest2.setQuestsTrackerLayout()
			end
		elseif info[#info-1] == "weightedLevel" then
			tomQuest2:computeQuestsLevel()
			tomQuest2:questsSort()
			tomQuest2.setQuestsTrackerLayout()
		elseif info[#info-1] == "questsTag" then
			tomQuest2:computeQuestsTag()
		elseif info[#info] == "zeroColor" then
			tomQuest2.db.profile.rj0, tomQuest2.db.profile.gj0, tomQuest2.db.profile.bj0 = arg1, arg2, arg3
			local rh, gh, bh = normalizeColorComponent(0.5*tomQuest2.db.profile.ri1 + tomQuest2.db.profile.rj1), normalizeColorComponent(0.5*tomQuest2.db.profile.gi1 + tomQuest2.db.profile.gj1), normalizeColorComponent(0.5*tomQuest2.db.profile.bi1 + tomQuest2.db.profile.bj1)
			computeFirstFunction(arg1, arg2, arg3, rh, gh, bh)
			computeQuestsObjectivesStatus()
		elseif info[#info] == "halfColor" then
			local r0, g0, b0 = normalizeColorComponent(tomQuest2.db.profile.rj0) , normalizeColorComponent(tomQuest2.db.profile.gj0), normalizeColorComponent(tomQuest2.db.profile.bj0)
			computeFirstFunction(r0, g0, b0, arg1, arg2, arg3)
			local r1, g1, b1 = normalizeColorComponent(tomQuest2.db.profile.rj3), normalizeColorComponent(tomQuest2.db.profile.gj3), normalizeColorComponent(tomQuest2.db.profile.bj3)
			computeSecondFunction(arg1, arg2, arg3, r1, g1, b1)
			computeQuestsObjectivesStatus()
		elseif info[#info] == "fullColor" then
			local rh, gh, bh = normalizeColorComponent(0.5*tomQuest2.db.profile.ri1 + tomQuest2.db.profile.rj1), normalizeColorComponent(0.5*tomQuest2.db.profile.gi1 + tomQuest2.db.profile.gj1), normalizeColorComponent(0.5*tomQuest2.db.profile.bi1 + tomQuest2.db.profile.bj1)
			computeSecondFunction(rh, gh, bh, arg1, arg2, arg3)
			tomQuest2.db.profile.rj3, tomQuest2.db.profile.gj3, tomQuest2.db.profile.bj3 = arg1, arg2, arg3
			computeQuestsObjectivesStatus()
		elseif info[#info-1] == "customColor" then
			computeQuestsObjectivesStatus()
		elseif info[#info] == "categoryWithoutLvlPos" then
			for zone, zData in pairs(tomQuest2.questsCategories) do
				if zData.minLevel == "" then
					tomQuest2.questsCategories[zone].sMinLevel = tomQuest2.db.profile.zoneWithoutLvlPos
				end
				if zData.maxLevel == "" then
					tomQuest2.questsCategories[zone].sMaxLevel = tomQuest2.db.profile.zoneWithoutLvlPos
				end
			end
			tomQuest2:questsSort()
			tomQuest2.setQuestsTrackerLayout()
		elseif info[#info-1] == "globalFont" then
			tomQuest2.db.profile[info[#info]] = arg1
			local suffix
			if info[#info] == "font" then
				suffix = ""
				tomQuest2.db.profile.qlFont = arg1
				tomQuest2.db.profile.lhFont = arg1
			elseif info[#info] == "fontSize" then
				suffix = "Size"
				tomQuest2.db.profile.qlFontSizeH1 = arg1
				tomQuest2.db.profile.qlFontSizeH2 = arg1
				tomQuest2.db.profile.qlFontSize = arg1
				tomQuest2.db.profile.lhFontSize = arg1
			elseif info[#info] == "fontOutline" then
				suffix = "Outline"
			end
			tomQuest2.db.profile.achievementsTracker["categoryFont"..suffix] = arg1
			tomQuest2.db.profile.achievementsTracker["subCategoryFont"..suffix] = arg1
			tomQuest2.db.profile.achievementsTracker["achievementFont"..suffix] = arg1
			tomQuest2.db.profile.achievementsTracker["achievementCriterionFont"..suffix] = arg1
			tomQuest2.db.profile.achievementsTracker["statusBarFont"..suffix] = arg1
			tomQuest2.db.profile.questsTooltip["categoryFont"..suffix] = arg1
			tomQuest2.db.profile.questsTooltip["questFont"..suffix] = arg1
			tomQuest2.db.profile.questsTooltip["questObjectiveFont"..suffix] = arg1
			tomQuest2.db.profile.questsTracker["categoryFont"..suffix] = arg1
			tomQuest2.db.profile.questsTracker["questFont"..suffix] = arg1
			tomQuest2.db.profile.questsTracker["questObjectiveFont"..suffix] = arg1
			globalFont()
		elseif info[#info-1] == "sorting" then
			tomQuest2:questsSort()
			tomQuest2.setQuestsTrackerLayout()
		elseif info[#info-1] == "counter" then
			if info[#info] == "showRemainingTime" then
				if arg1 then
					tomQuest2.dailyTimer = tomQuest2:ScheduleRepeatingTimer("updateDQToText", 60)
				else
					tomQuest2:CancelTimer(tomQuest2.dailyTimer, true)
					tomQuest2.dailyTimer = nil
				end
			end
			tomQuest2:updateDQToText()
		elseif info[#info] == "hideBlizzardNotification" then
				tomQuest2:setBlizzNotification()
		elseif info[#info] == "questTitleColor" then
			tomQuest2.db.profile[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
		end
	end
end
-------------------------
--- Main Option table ---
-------------------------
local options = {type = "group", childGroups = "tab", name = "tomQuest2", get = optGetter, set = optSetter,
	args = {
		modules = {type = "group", name = LCo["Modules"], order = 1,
			args = {
				syncParty = {type = "toggle", name = LCo["Party syncing"], width = "double", order = 1},
				achievementsTracker = {type = "toggle", name = LCo["Achievements tracker"], width = "double", order = 2},
				questsTracker = {type = "toggle", name = LCo["Quests tracker"], width = "double", order = 3},
				questsTooltip = {type = "toggle", name = LCo["Quests tooltip"], width = "double", order = 4},
				dailyTooltip = {type = "toggle", name = LCo["Daily tooltip"], width = "double", order = 5},
				informations = {type = "toggle", name = LCo["Information"], width = "double", order = 6},
				notifications = {type = "toggle", name = LCo["Notifications"], width = "double", order = 7},
				soundNotifications = {type = "toggle", name = LCo["Sound notifications"], width = "double", order = 8},
				tagQuestLog = {type = "toggle", name = LCo["Tag quests log"], width = "double", order = 9},
				questsGivers = {type = "toggle", name = LCo["Quests givers"], width = "double",order = 10},
				tooltips = {type = "toggle", name = LCo["Tooltips"], width = "double", order = 11},
				achievementsTradeSkills = {type = "toggle", name = LCo["Achievements trade skills"], width = "double", order = 12},
				announce = {type = "toggle", name = LCo["Announce"], width = "double", order = 13},
			},
		},
		raidMode = {type = "group", name = LCo["Raid mode"], order = 2,
			args = {
				enabled = {type = "toggle", name = LCo["Raid mode"], width = "full", order = 1},
				raidModeDesc = {type = "description", name = LCo["Define module state when in a raid"], order = 2},
				syncParty = {type = "select", values = raidModeValues, name = LCo["Party syncing"], width = "double", order = 3},
				achievementsTracker = {type = "select", values = raidModeTrackerValues, name = LCo["Achievements tracker"], width = "double", order = 4},
				questsTracker = {type = "select", values = raidModeTrackerValues, name = LCo["Quests tracker"], width = "double", order = 5},
				questsTooltip = {type = "select", values = raidModeValues, name = LCo["Quests tooltip"], width = "double", order = 6},
				dailyTooltip = {type = "select", values = raidModeValues, name = LCo["Daily tooltip"], width = "double", order = 7},
				informations = {type = "select", values = raidModeValues, name = LCo["Information"], width = "double", order = 8},
				notifications = {type = "select", values = raidModeValues, name = LCo["Notifications"], width = "double", order = 9},
				soundNotifications = {type = "select", values = raidModeValues, name = LCo["Sound notifications"], width = "double", order = 10},
				tagQuestLog = {type = "select", values = raidModeValues, name = LCo["Tag quests log"], width = "double", order = 11},
				questsGivers = {type = "select", values = raidModeValues, name = LCo["Quests givers"], width = "double", order = 12},
				tooltips = {type = "select", values = raidModeValues, name = LCo["Tooltips"], width = "double", order = 13},
				achievementsTradeSkills = {type = "select", values = raidModeValues, name = LCo["Achievements trade skills"], width = "double", order = 14},
				announce = {type = "select", values = raidModeValues, name = LCo["Announce"], width = "double", order = 15},
			}
		},
		globalFont = {type = "group", name = LCo["Global font"], order = 3,
			args = {
				font = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
				fontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
				fontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
			},
		},
		other = {type = "group", name = LCo["Other"], order = 4,
			args = {
				partyHeader = {name = LCo["Show all party quests you don't have under a specific header"], type = "toggle", width = "full", order = 1},
				maximumTooltipWidth = {name = LCo["Maximum tooltip width"], type = "range", min = 50, max = 500, step = 10, width = "full", order = 2},
				hideDefaultBlizTracker = {name = LCo["Hide default Blizzard quests and achievements tracker"], type = "toggle", width = "full", order = 3},
				macro = {name = LCo["Macro Generator"], type = "execute", func = specialItemMacro, order = 4},
				ldbInteraction = {type = "group", name = LCo["Quests and daily quests counter interaction"], inline = true, order = 5,
					args = {
						dataObjMouseRightShift = {name = LCo["Action on right shift click"], type = "select", values = ldbInteractivityList, width = "full", order = 1},
						dataObjMouseRightControl = {name = LCo["Action on right control click"], type = "select", values = ldbInteractivityList, width = "full", order = 2},
						dataObjMouseRightAlt = {name = LCo["Action on right alt click"], type = "select", values = ldbInteractivityList, width = "full", order = 3},
						dataObjMouseRight = {name = LCo["Action on right click"], type = "select", values = ldbInteractivityList, width = "full", order = 4},
						dataObjMouseLeftShift = {name = LCo["Action on left shift click"], type = "select", values = ldbInteractivityList, width = "full", order = 5},
						dataObjMouseLeftControl = {name = LCo["Action on left control click"], type = "select", values = ldbInteractivityList, width = "full", order = 6},
						dataObjMouseLeftAlt = {name = LCo["Action on left alt click"], type = "select", values = ldbInteractivityList, width = "full", order = 7},
						dataObjMouseLeft = {name = LCo["Action on left click"], type = "select", values = ldbInteractivityList, width = "full", order = 8},
						dataObjMouseMiddleShift = {name = LCo["Action on middle shift click"], type = "select", values = ldbInteractivityList, width = "full", order = 9},
						dataObjMouseMiddleControl = {name = LCo["Action on middle control click"], type = "select", values = ldbInteractivityList, width = "full", order = 10},
						dataObjMouseMiddleAlt = {name = LCo["Action on middle alt click"], type = "select", values = ldbInteractivityList, width = "full", order = 11},
						dataObjMouseMiddle = {name = LCo["Action on middle click"], type = "select", values = ldbInteractivityList, width = "full", order = 12},
					},
				},
			},
		},
		weightedLevel = {type = "group", name = LCo["Weighted quest level computation"], order = 5,
			args = {
				eliteAddLevel = {name = LCo["Level to add for an elite quest"], type = "range", min = 0, max = 5, step = 1, width = "full", order = 1},
				groupAddLevel = {name = LCo["Level to add per missing player for a group quest"], type = "range", min = 0, max = 5, step = 1, width = "full", order = 2},
				dungeonAddLevel = {name = LCo["Level to add per missing player for a dungeon quest"], type = "range", min = 0, max = 5, step = 1, width = "full", order = 3},
				heroicAddLevel = {name = LCo["Level to add per missing player for an heroic quest"], type = "range", min = 0, max = 5, step = 1, width = "full", order = 4},
				raidAddLevel = {name = LCo["Level to add per missing player for a raid quest"], type = "range", min = 0, max = 5, step = 1, width = "full", order = 5},
			},
		},
		questsTag = {type = "group", name = LCo["Custom quests tag"], order = 6,
			args = {
				level = {name = LCo["Quest level tag"], type = "select", values = questLevelTag, width = "full", order = 1, },
				elite = {name = LCo["Elite quest tag"], type = "input", order = 2},
				group = {name = LCo["Group quest tag"], type = "input", order = 3},
				pvp = {name = LCo["PVP quest tag"], type = "input", order = 4},
				raid = {name = LCo["Raid quest tag"], type = "input", order = 5},
				dungeon = {name = LCo["Dungeon quest tag"], type = "input", order = 6},
				heroic = {name = LCo["Heroic quest tag"], type = "input", order = 7},
				daily = {name = LCo["Daily quest tag"], type = "input", order = 8},
				escort = {name = LCo["Escort quest tag"], type = "input", order = 9},
			},
		},
		customColor = {type = "group", name = LCo["Custom objective coloring function"], order = 7,
			args = {
				easyColorSetupDesc = {type = "description", name = LCo["You can define a color for percentage of completion = 0, 50% and 100%. tomQuest2 will automatically define the needed color function between those threee points"], width = "full", order = 0},
				zeroColor = {name = LCo["Color for completion = 0%"], type = "color", hasAlpha = false, width = "full", order = 1},
				halfColor = {name = LCo["Color for completion = 50%"], type = "color", hasAlpha = false, width = "full", order = 2},
				fullColor = {name = LCo["Color for completion = 100%"], type = "color", hasAlpha = false, width = "full", order = 3},
				resetColor = {name = LCo["Reset color functions"], type = "execute", func = setDefaultColor, width = "full", order = 4},
				customColorMode = {name = LCo["Hide custom color function setup"], type = "toggle", width = "full", order = 5},
				customColorFunctionDesc = {type = "description", name = LCo["Custom objective coloring allows you to define multiplicative and additive component of the coloring function. For each color component you'll have a function that is color = (multiplicative component) * percent + (additive component) where percent is the percent of completion of the objective."], width = "full", hidden = hideCustomColorMode, order = 6},
				zeroCustomColorFunctionHeader = {type = "header", name = LCo["Function for objective completion at 0%"], width = "full", hidden = hideCustomColorMode, order = 7},
				rj0 = {name = LCo["Red add."], type = "range", min = 0, max = 1, step = 0.1, hidden = hideCustomColorMode, order = 8},
				gj0 = {name = LCo["Green add."], type = "range", min = 0, max = 1, step = 0.1, hidden = hideCustomColorMode, order = 9},
				bj0 = {name = LCo["Blue add."], type = "range", min = 0, max = 1, step = 0.1, hidden = hideCustomColorMode, order = 10},
				firstCustomColorFunctionHeader = { type = "header", name = LCo["Function for [0%, 50%[ objective completion"], hidden = hideCustomColorMode, order = 11},
				ri1 = {name = LCo["Red mult."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 12},
				gi1 = {name = LCo["Green mult."],type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 13},
				bi1 = {name = LCo["Blue mult."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 14},
				rj1 = {name = LCo["Red add."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 15},
				gj1 = {name = LCo["Green add."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 16},
				bj1 = {name = LCo["Blue add."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 17},
				secondCustomColorFunctionHeader = {type = "header", name = LCo["Function for ]50%, 100%[ objective completion"], hidden = hideCustomColorMode, order = 18},
				ri2 = {name = LCo["Red mult."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 19},
				gi2 = {name = LCo["Green mult."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 20},
				bi2 = {name = LCo["Blue mult."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 21},
				rj2 = {name = LCo["Red add."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 22},
				gj2 = {name = LCo["Green add."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 23},
				bj2 = {name = LCo["Blue add."], type = "range", min = -5, max = 5, step = 0.1, hidden = hideCustomColorMode, order = 24},
				fullCustomColorFunctionHeader = {type = "header", name = LCo["Function for objective completion at 100%"], hidden = hideCustomColorMode, order = 25},
				rj3 = {name = LCo["Red add."], type = "range", min = 0, max = 1, step = 0.1, hidden = hideCustomColorMode, order = 26},
				gj3 = {name = LCo["Green add."], type = "range", min = 0, max = 1, step = 0.1, hidden = hideCustomColorMode, order = 27},
				bj3 = {name = LCo["Blue add."], type = "range", min = 0, max = 1, step = 0.1, hidden = hideCustomColorMode, order = 28},
			},
		},
		questStatusColor = {type = "group", name = LCo["Quest status color"], order = 8,
			args = {
				completedColor = {name = LCo["Color for quest complete"], type = "color", hasAlpha = false, width = "full", order = 1},
				failedColor = {name = LCo["Color for quest failed"], type = "color", hasAlpha = false, width = "full", order = 2},
			},
		},
		sorting = {type = "group", name = LCo["Zones and quests sorting"], order = 9,
			args = {
				categorySorting = {type = "select", name = LCo["Zones sorting method"], values = zSorting, width = "full", order = 1},
				categoryWithoutLvlPos = {type = "select", name = LCo["Position of zones without level when sorting zones by level"], values = zwlPosition, width = "full", order = 2},
				questsSorting = {type = "select", name = LCo["Quests sorting method"], values = qSorting, width = "full", order = 3},
			},
		},
	}
}

----------------------------------------
--- Sound Notifications option table ---
----------------------------------------
local soundNotificationsOptions = {type = "group", name = LCo["Sound notifications"], get = optGetter, set = optSetter,
	args = {
		soundObjectiveComplete = {name = LCo["Quest objective complete sound"], type = "select", dialogControl = "LSM30_Sound", values = AceGUIWidgetLSMlists.sound, width = "full", order = 1},
		soundQuestComplete = {name = LCo["Quest complete sound"], type = "select", dialogControl = "LSM30_Sound", values = AceGUIWidgetLSMlists.sound, width = "full", order = 2},
		soundQuestAutoComplete = {name = LCo["Quest auto complete sound"], type = "select", dialogControl = "LSM30_Sound", values = AceGUIWidgetLSMlists.sound, width = "full", order = 3},
		soundQuestFailed = { name = LCo["Quest failed sound"], type = "select", dialogControl = "LSM30_Sound", values = AceGUIWidgetLSMlists.sound, width = "full", order = 4},
	},
}

-----------------------------
--- Announce option table ---
-----------------------------
local announceOptions = {type = "group", name = LCo["Announce"], get = optGetter, set = optSetter,
	args = {
		announcePlayerObjectiveProgress = {name = LCo["Quests objectives progress annoucement"], type = "toggle", width = "full", order = 1},
		announcePlayerObjectiveComplete = {name = LCo["Quests objectives complete announcement"], type = "toggle", width = "full", order = 2},
		announcePlayerQuestComplete = {name = LCo["Quests complete announcement"], type = "toggle", width = "full", order = 3},
		announcePlayerQuestFailed = {name = LCo["Quests failed announcement"], type = "toggle", width = "full", order = 4},
		announceObjectiveProgressWhenQuestComplete = {name = LCo["Objective announce on quest completion"], type = "toggle", width = "full", order = 5}
	},
}

----------------------------------
--- Daily tooltip option table ---
----------------------------------
local dailyTooltipOptions = {type = "group", childGroups = "tab", name = LCo["Daily tooltip"], get = optGetter, set = optSetter,
	args = {
		counter = {type = "group", name = LCo["Daily quests counter"],
			args = {
				showNumCompletedDailyQuests = {name = LCo["Show number of completed daily quest"], type = "toggle", width = "full", order = 1},
				showMaxDailyQuest = {name = LCo["Show maximum daily quests number"], type = "toggle", width = "full", order = 2},
				showRemainingTime = {name = LCo["Show remaining time before end of day"], type = "toggle", width = "full", order = 3},
				colorizeDailyQuestsCounter = {name = LCo["Color daily quests counter"], type = "toggle", width = "full", order = 4},
				colorizeDailyQuestsTimer = {name = LCo["Color daily quests timer"], type = "toggle", width = "full", order = 5},
			},
		},
		tooltip = {type = "group", name = LCo["Daily tooltip"],
			args = {
				showCOTD = {name = LCo["Show in the tooltip daily quests completed today"], type = "toggle", width = "full", order = 1},
			},
		},
	},
}

----------------------------------
--- Notifications option table ---
----------------------------------
local notificationsOptions = {type = "group", childGroups = "tab", name = LCo["Notifications"], get = optGetter, set = optSetter,
	args = {
		common = {type = "group", name = LCo["Common settings"],
			args = {
				hideBlizzardNotification = {name = LCo["Hide default Blizzard quests notification"], type = "toggle", width = "full", order = 1},
				notifiyObjectiveProgressWhenQuestComplete = {name = LCo["Objective notification on quest completion"], type = "toggle", width = "full", order = 2},
			},
		},
		player = {type = "group", name = LCo["Player notifications"],
			args = {
				notifyPlayerObjectiveProgress = {name = LCo["Quest objective progress notification"], type = "toggle", width = "full", order = 1},
				notifyPlayerObjectiveComplete = {name = LCo["Quest objective complete notification"], type = "toggle", width = "full", order = 2},
				notifyPlayerQuestComplete = {name = LCo["Quest complete notification"], type = "toggle", width = "full", order = 3},
				notifyPlayerQuestAutoComplete = {name = LCo["Quest auto complete notification"], type = "toggle", width = "full", order = 4},
				notifyPlayerQuestFailed = {name = LCo["Quest failed notification"], type = "toggle", width = "full", order = 5},
				notifyItemStartingQuestLooted = {name = LCo["Item starting a quest notification"], type = "toggle", width = "full", order = 6},
			},
		},
		party = {type = "group", name = LCo["Party notifications"],
			args = {
				notifyPartyObjectiveProgress = {name = LCo["Quest objective progress notification"], type = "toggle", width = "full", order = 1},
				notifyPartyObjectiveComplete = {name = LCo["Quest objective complete notification"], type = "toggle", width = "full", order = 2},
				notifyPartyQuestComplete = {name = LCo["Quest complete notification"], type = "toggle", width = "full", order = 3},
				notifyPartyQuestFailed = {name = LCo["Quest failed notification"], type = "toggle", width = "full", order = 4},
				notifyPartyQuestAccepted = {name = LCo["Quest accepted notification"], type = "toggle", width = "full", order = 5},
				notifyPartyQuestTurnedIn = {name = LCo["Quest turned in notification"], type = "toggle", width = "full", order = 6},
				notifyPartyQuestAbandonned = {name = LCo["Quest abandonned notification"], type = "toggle", width = "full", order = 7},
			},
		},
		output = tomQuest2:GetSinkAce3OptionsDataTable()
	},
}

----------------------------------
--- Quests Givers option table ---
----------------------------------
local questsGiversOptions = {type = "group", childGroups = "tab", name = LCo["Quests givers"], get = optGetter, set = optSetter,
	args = {
		questTitleColor = {name = LCo["Quest title color"], type = "color", hasAlpha = false, width = "full", order = 1},
		addQuestLevel = {name = LCo["Add quest level to the quest name"], type = "toggle", width = "full", order = 2},
		fadeNotCompletedQuests = {name = LCo["Fade quest name that are not complete"], type = "toggle", width = "full", order = 3},
		automaticAccept = {name = LCo["Accept quests automatically"], type = "select", values = automaticAcceptChoices, width = "full", order = 4},
		automaticAcceptEscort = {name = LCo["Accept escort quest started by other player automatically"], type = "toggle", width = "full", order = 5},
		automaticTurnIn = {name = LCo["Automatic quest turn-in"], type = "toggle", width = "full", order = 6},
	},
}

--------------------------------------
--- Quets Information option table ---
--------------------------------------
local function questsInformationsGetter(info)
	if info[#info] == "backDropColor" or info[#info] == "borderColor" then
		return unpack(tomQuest2.db.profile[info[#info]])
	else
		return tomQuest2.db.profile[info[#info]]
	end
end
local function questsInformationsSetter(info, arg1, arg2, arg3, arg4)
	if info[#info] == "backDropColor" or info[#info] == "borderColor" then
		tomQuest2.db.profile[info[#info]][1], tomQuest2.db.profile[info[#info]][2], tomQuest2.db.profile[info[#info]][4], tomQuest2.db.profile[info[#info]][4] = arg1, arg2, arg3, arg4
		_G["tomQuest2ParentFrame"]:SetBackdropColor(tomQuest2.db.profile.backDropColor[1], tomQuest2.db.profile.backDropColor[2], tomQuest2.db.profile.backDropColor[3], tomQuest2.db.profile.backDropColor[4])
		_G["tomQuest2ParentFrame"]:SetBackdropBorderColor(tomQuest2.db.profile.borderColor[1], tomQuest2.db.profile.borderColor[2], tomQuest2.db.profile.borderColor[3], tomQuest2.db.profile.borderColor[4])
	else
		tomQuest2.db.profile[info[#info]] = arg1
	end
	tomQuest2.refreshQuestInformation()
end
local questsInformationsOptions = {type = "group", childGroups = "tab", name = LCo["Information"], get = questsInformationsGetter, set = questsInformationsSetter,
	args = {
		common = {type = "group", name = LCo["Common settings"],
			args = {
				backDropColor = {name = LCo["Background color"], type = "color", hasAlpha = true, width = "full", order = 1},
				borderColor = {name = LCo["Border color"], type = "color", hasAlpha = true, width = "full", order = 2},
				attach = {name = LCo["Attach LightHeaded to quests log"], type = "select", values = attachSelect, width = "full", order = 3},
			},
		},
		questLog = {type = "group", name = LCo["Quests log settings"],
			args = {
				showQuestLog = {name = LCo["Show tomQuest2 quest log"], type = "toggle", width = "full", order = 1},
				qlFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 2},
				qlFontSizeH1 = {name = LCo["Quest title font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 3},
				qlFontSizeH2 = {name = LCo["Headers font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 4},
				qlFontSize = {name = LCo["Text font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 5},
			},
		},
		lightHeaded = {type = "group", name = LCo["LightHeaded settings"],
			args = {
				showLightHeaded = {name = LCo["Show LightHeaded into tomQuest2"], type = "toggle", width = "full", order = 1},
				showAllCommentsOnOnePage = {name = LCo["Show all LightHeaded comments on the same page"], type = "toggle", width = "full", order = 2},
				lhFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 3},
				lhFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 4},
			},
		},
	},
}
-----------------------------
--- tooltips option table ---
-----------------------------
local function optTooltipSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if not tomQuest2.moduleState["tooltips"] then return end
	if info[#info] == "mobTooltip" then
		tomQuest2.setMobTooltip()
	else
		tomQuest2.setItemTooltip()
	end
end
local tooltipsOptions = {type = "group", name = LCo["Tooltips"], get = optGetter, set = optTooltipSetter,
	args = {
		mobTooltip = {name = LCo["Mobs tooltip modification"], type = "toggle", width = "full", order = 1},
		itemTooltip = {name = LCo["Item tooltip modification"], type = "toggle", width = "full", order = 2},
	},
}
-----------------------------------
--- Quests Tooltip option table ---
-----------------------------------
local function questsTooltipOptGetter(info)
	if info[#info-1] == "questsCounter" then
		return tomQuest2.db.profile[info[#info]]
	else
		return tomQuest2.db.profile.questsTooltip[info[#info]]
	end
end
local function questsTooltipOptSetter(info, value)
	if info[#info-1] == "questsCounter" then
		tomQuest2.db.profile[info[#info]] = value
		tomQuest2:updateQuestCounter()
	else
		tomQuest2.db.profile.questsTooltip[info[#info]] = value
		if info[#info-1] == "category" then
			_G["cQToFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.categoryFont), tomQuest2.db.profile.questsTooltip.categoryFontSize, tomQuest2.db.profile.questsTooltip.categoryFontOutline)
		elseif info[#info-1] == "quest" then
			_G["qQToFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.questFont), tomQuest2.db.profile.questsTooltip.questFontSize, tomQuest2.db.profile.questsTooltip.questFontOutline)
		elseif info[#info-1] == "questObjective" then
			_G["qoQToFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTooltip.questObjectiveFont), tomQuest2.db.profile.questsTooltip.questObjectiveFontSize, tomQuest2.db.profile.questsTooltip.questObjectiveFontOutline)
		elseif info[#info-1] == "backdrop" then
			tomQuest2.setBackDrop("questsTooltip")
		elseif info[#info] == "hideZoneLevelRange" then
			for category in pairs(tomQuest2.questsCategories) do
				tomQuest2.updateQuestCategoryCell(category, "questsTooltip")
			end
		elseif info[#info] == "hideQuestTag" then
			for qid in pairs(tomQuest2.quests) do
				tomQuest2.updateQuestTextCell(qid, "questsTooltip")
			end
		elseif info[#info] == "collapseButtonSize" then
			for _, categoryData in pairs(tomQuest2.questsCategories) do
				categoryData.questsTooltipCollapse:SetScale(value/categoryData.questsTooltipCollapse.height)
			end
		elseif info[#info] == "questItemIconSize" then
			for _, questData in pairs(tomQuest2.quests) do
				if questData.questsTooltipSpecialItem then
					questData.questsTooltipSpecialItem:SetScale(value/questData.questsTooltipSpecialItem.height)
				elseif questData.questsTooltipFakeItem then
					questData.questsTooltipFakeItem:SetScale(value/questData.questsTooltipFakeItem.height)
				end
			end
		end
	end
end

local function questsTooltipColorGetter(info)
	return unpack(tomQuest2.db.profile.questsTooltip[info[#info]])
end
local function questsTooltipColorSetter(info, r, g, b, a)
	tomQuest2.db.profile.questsTooltip[info[#info]][1] = r
	tomQuest2.db.profile.questsTooltip[info[#info]][2] = g
	tomQuest2.db.profile.questsTooltip[info[#info]][3] = b
	tomQuest2.db.profile.questsTooltip[info[#info]][4] = a
end
local function setQuestsTooltipColors(info, value)
	tomQuest2.db.profile.questsTooltip[info[#info]] = value
	if info[#info] == "colorizeCategoryName" or info[#info] == "colorizeCategoryLevel" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2.updateQuestCategoryCell(category, "questsTooltip")
		end
	elseif info[#info] == "colorizeQuestName" or info[#info] == "colorizeQuestLevel" or info[#info] == "colorizeQuestTimer"then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2.updateQuestTextCell(qid, "questsTooltip")
		end
	elseif info[#info] == "colorizeQuestObjective" or info[#info] == "colorizeQuestObjectiveCounter" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2.updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTooltip[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTooltip.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTooltip[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTooltip.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor)
			end
		end
	end
end
local function questsTooltipHexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile.questsTooltip[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTooltip[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTooltip[info[#info]], 5, 6), 16) / 255
end
local function questsTooltipHexColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile.questsTooltip[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	if info[#info] == "defaultCategoryNameColor" or info[#info] == "defaultCategoryLevelColor" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2.updateQuestCategoryCell(category, "questsTooltip")
		end
	elseif info[#info] == "defaultQuestNameColor" or info[#info] == "defaultQuestLevelColor" or info[#info] == "defaultQuestTimerColor" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2.updateQuestTextCell(qid, "questsTooltip")
		end
	elseif info[#info] == "defaultQuestObjectiveColor" or info[#info] == "defaultQuestObjectiveCounterColor" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2.updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTooltip[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTooltip.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTooltip[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTooltip.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTooltip.defaultQuestObjectiveColor)
			end
		end
	elseif info[#info] == "defaultQuestDescriptionColor" then
		for qid, qData in pairs(tomQuest2.quests) do
			if qData.questsTooltipDescription then
				qData.questsTooltipDescription.fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTooltip.defaultQuestDescriptionColor, qData.objective)
			end
		end
	end
end
local questsTooltipOptions = {type = "group", childGroups = "tab", name = LCo["Quests tooltip"], get = questsTooltipOptGetter, set = questsTooltipOptSetter,
	args = {
		questsCounter = {type = "group", name = LCo["Quests counter"], 
			args = {
				showNumCompletedQuests = {name = LCo["Show number of completed quest"], type = "toggle", width = "full", order = 1},
				showNumQuests = {name = LCo["Show number of quests"], type = "toggle", width = "full", order = 2},
				showMaxQuest = {name = LCo["Show maximum quests number"], type = "toggle", width = "full", order = 3},
				colorizeQuestsCounter = {name = LCo["Color quests counter"], type = "toggle", width = "full", order = 4},
			},
		},
		display = {type = "group", name = LCo["Display"],
			args = {
				hideCompletedObjective = {name = LCo["Hide completed quests objectives"], type = "select", values = hideObjectives, width = "full", order = 1},
				hideZoneLevelRange = {name = LCo["Hide zone level range"], type = "toggle", width = "full", order = 2},
				hideQuestTag = {name = LCo["Hide quest tag"], type = "toggle", width = "full", order = 3},
			},
		},
		fonts = {type = "group", name = LCo["Tooltip fonts"],
			args = {
				category = {type = "group", name = LCo["Zone"], inline = true, order = 1,
					args = {
						categoryFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						categoryFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						categoryFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				quest = {type = "group", name = LCo["Quests"], inline = true, order = 2,
					args = {
						questFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						questFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						questFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				questObjective = {type = "group", name = LCo["Quests objectives"], inline = true, order = 3,
					args = {
						questObjectiveFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						questObjectiveFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						questObjectiveFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
			},
		},
		appearance = {type = "group", name = LCo["Tooltip appearance"],
			args = {
				backdrop = {type = "group", name = LCo["Backdrop"], inline = true, order = 1,
					args = {
						bgTexture = {name = LCo["Background texture"], type = "select", dialogControl = "LSM30_Background", values = AceGUIWidgetLSMlists.background, order = 1},
						borderTexture = {name = LCo["Border texture"], type = "select", dialogControl = "LSM30_Border", values = AceGUIWidgetLSMlists.border, order = 2},
						borderWidth = {name = LCo["Border width"], type = "range", min = 1, max = 24, step = 1, order = 3},
						borderInset = {name = LCo["Inset"], type = "range", min = 0, max = 10, step = 1, order = 4},
						backDropColor = {name = LCo["Quests tooltip background color"], type = "color", hasAlpha = true, get = questsTooltipColorGetter, set = questsTooltipColorSetter, width = "full", order = 5},
						borderColor = {name = LCo["Quests tooltip border color"], type = "color", hasAlpha = true, get = questsTooltipColorGetter, set = questsTooltipColorSetter, width = "full", order = 6},
					}
				},
				scale = {name = LCo["Quests tooltip scale"], type = "range", min = 0, max = 1, step = 0.1, width = "full", order = 7},
				hasMaxHeight = {name = LCo["Set quests tooltip maximum height after which it will be scrollable"], type = "toggle", width = "full", order = 8},
				maxHeight = {name = LCo["Quests tooltip max height"], type = "range", min = 100, max = 2000, step = 5, width = "full", order = 9},
				showCollapseButton = {name = LCo["Show collapse button"], type = "toggle", width = "full", order = 10},
				collapseButtonSize = {name = LCo["Collapse button size"], type = "range", min = 10, max = 40, step = 1, width = "full", order = 11},
				questsCellPadding = {name = LCo["Quests cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 12},
				objectivesCellPadding = {name = LCo["Quests objectives cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 13},
				maxWidth = {name = LCo["Quests Tooltip column max width"], type = "range", min = 50, max = 600, step = 5, width = "full", order = 14},
				minWidth = {name = LCo["Quests Tooltip column min width"], type = "range", min = 50, max = 600, step = 5, width = "full", order = 15},
				statusMaxWidth = {name = LCo["Quests Tooltip status column max width"], type = "range", min = 30, max = 200, step = 5, width = "full", order = 16},
				statusMinWidth = {name = LCo["Quests Tooltip status column min width"], type = "range", min = 30, max = 200, step = 5, width = "full", order = 17},
			},
		},
		colors = {type = "group", name = LCo["Colors"],
			args = {
				colorizeCategoryName = {name = LCo["Color quests header name"], type = "toggle", set = setQuestsTooltipColors, width = "full", order = 3},
				defaultCategoryNameColor = {name = LCo["Default header name color"], type = "color", hasAlpha = false, get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 4},
				colorizeCategoryLevel = {name = LCo["Color quests header level"], type = "toggle", set = setQuestsTooltipColors, width = "full", order = 5},
				defaultCategoryLevelColor = {name = LCo["Default header level color"], type = "color", hasAlpha = false, get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 6},
				colorizeQuestName = {name = LCo["Color quests name"], type = "toggle", set = setQuestsTooltipColors, width = "full", order = 7},
				defaultQuestNameColor = {name = LCo["Default quest name color"], type = "color", hasAlpha = false, get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 8},
				colorizeQuestLevel = {name = LCo["Color quests level"], type = "toggle", set = setQuestsTooltipColors, width = "full", order = 9},
				defaultQuestLevelColor = {name = LCo["Default quest level color"], type = "color", hasAlpha = false, get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 10},
				colorizeQuestObjective = {name = LCo["Color quests objectives"], type = "toggle", set = setQuestsTooltipColors, width = "full", order = 11},
				defaultQuestObjectiveColor = {name = LCo["Default quest objective color"], type = "color", hasAlpha = false, get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 12},
				colorizeQuestObjectiveCounter = {name = LCo["Color quests objectives counter"], type = "toggle", set = setQuestsTooltipColors, width = "full", order = 13},
				defaultQuestObjectiveCounterColor = {name = LCo["Default quest objective counter color"], type = "toggle", get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 14},
				defaultQuestDescriptionColor = {name = LCo["Default quest description color for quest without objective"], type = "color", hasAlpha = false, get = questsTooltipHexColorGetter, set = questsTooltipHexColorSetter, width = "full", order = 15},
			},
		},
		questsItems = {type = "group", name = LCo["Quests items"],
			args = {
				showQuestItemIcon = {name = LCo["Show special quest item icon behind quest name"], type = "toggle", width = "full", order = 1},
				questItemIconSize = {name = LCo["Quest item icon size"], type = "range", min = 10, max = 40, step = 1, width = "full", order = 2},
			},
		},
		tooltipInteraction = {type = "group", name = LCo["Tooltip interaction"],
			args = {
				showHints = {name = LCo["Show interractivity hints at the bottom of the tooltip"], type = "toggle", width = "full", order = 1},
				overTooltip = {name = LCo["Display quest tooltip on mouse over"], type = "toggle", width = "full", order = 2},
				mouseRightShift = {name = LCo["Action on right shift click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 3},
				mouseRightControl = {name = LCo["Action on right control click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 4},
				mouseRightAlt = {name = LCo["Action on right alt click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 5},
				mouseRight = {name = LCo["Action on right click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 6},
				mouseLeftShift = {name = LCo["Action on left shift click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 7},
				mouseLeftControl = {name = LCo["Action on left control click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 8},
				mouseLeftAlt = {name = LCo["Action on left alt click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 9},
				mouseLeft = {name = LCo["Action on left click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 10},
				mouseMiddleShift = {name = LCo["Action on middle shift click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 11},
				mouseMiddleControl = {name = LCo["Action on middle control click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 12},
				mouseMiddleAlt = {name = LCo["Action on middle alt click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 13},
				mouseMiddle = {name = LCo["Action on middle click"], type = "select", values = tomQuest2.interactivityList, width = "full", order = 14},
			},
		},
	},
}

-----------------------------------
--- Quests Tracker option table ---
-----------------------------------
local function questsTrackerOverrideAnchorSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	setTrackerpoint(_G['tomQuest2QuestsTracker'])
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerCategoryFontSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	_G["cQTrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.categoryFont), tomQuest2.db.profile.questsTracker.categoryFontSize, tomQuest2.db.profile.questsTracker.categoryFontOutline)
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerQuestFontSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	_G["qQTrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.questFont), tomQuest2.db.profile.questsTracker.questFontSize, tomQuest2.db.profile.questsTracker.questFontOutline)
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerQuestObjectiveFontSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	_G["qoQTrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.questsTracker.questObjectiveFont), tomQuest2.db.profile.questsTracker.questObjectiveFontSize, tomQuest2.db.profile.questsTracker.questObjectiveFontOutline)
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerOptGetter(info)
	return tomQuest2.db.profile.questsTracker[info[#info]]
end
local function questsTrackerOptSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2.setQuestsTrackerLayout()
end
local function byCategorySetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	if not value then
		for _, cData in pairs(tomQuest2.questsCategories) do
			cData.questsTracker:Hide()
		end
	end
	tomQuest2.setQuestsTrackerLayout()
end
local function lockQuestsTrackerSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	_G["tomQuest2QuestsTracker"]:SetMovable(not value)
end
local function hZLOptSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for category in pairs(tomQuest2.questsCategories) do
		tomQuest2.updateQuestCategoryCell(category, "questsTracker")
	end
	tomQuest2.setQuestsTrackerLayout()
end
local function hQTOptSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for qid in pairs(tomQuest2.quests) do
		tomQuest2.updateQuestTextCell(qid, "questsTracker")
	end
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerBackdropSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	tomQuest2.setBackDrop("questsTracker")
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerCollapseButtonSizeSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for _, qcData in pairs(tomQuest2.questsCategories) do
		qcData.questsTrackerCollapse:SetScale(value/qcData.questsTrackerCollapse.height)
	end
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerColorGetter(info)
	return unpack(tomQuest2.db.profile.questsTracker[info[#info]])
end
local function questsTrackerColorSetter(info, r, g, b, a)
	tomQuest2.db.profile.questsTracker[info[#info]][1] = r
	tomQuest2.db.profile.questsTracker[info[#info]][2] = g
	tomQuest2.db.profile.questsTracker[info[#info]][3] = b
	tomQuest2.db.profile.questsTracker[info[#info]][4] = a
	tomQuest2.setQuestsTrackerLayout()
end
local function setquestsTrackerColors(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	if info[#info] == "colorizeCategoryName" or info[#info] == "colorizeCategoryLevel" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2.updateQuestCategoryCell(category, "questsTracker")
		end
	elseif info[#info] == "colorizeQuestName" or info[#info] == "colorizeQuestLevel" or info[#info] == "colorizeQuestTimer" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2.updateQuestTextCell(qid, "questsTracker")
		end
	elseif info[#info] == "colorizeQuestObjective" or info[#info] == "colorizeQuestObjectiveCounter" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2.updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTracker[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTracker.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTracker[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTracker.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor)
			end
		end
	end
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerHexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile.questsTracker[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTracker[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile.questsTracker[info[#info]], 5, 6), 16) / 255
end
local function questsTrackerHexColorSetter(info, arg1, arg2, arg3)
	tomQuest2.db.profile.questsTracker[info[#info]] = format("%02x%02x%02x", arg1*255, arg2*255, arg3*255)
	if info[#info] == "defaultCategoryNameColor" or info[#info] == "defaultCategoryLevelColor" then
		for category in pairs(tomQuest2.questsCategories) do
			tomQuest2.updateQuestCategoryCell(category, "questsTracker")
		end
	elseif info[#info] == "defaultQuestNameColor" or info[#info] == "defaultQuestLevelColor" or info[#info] == "defaultQuestTimerColor" then
		for qid in pairs(tomQuest2.quests) do
			tomQuest2.updateQuestTextCell(qid, "questsTracker")
		end
	elseif info[#info] == "defaultQuestObjectiveColor" or info[#info] == "defaultQuestObjectiveCounterColor" then
		for qid, qoData in pairs(tomQuest2.questsObjectives) do
			for oid in pairs(qoData) do
				local qoStatus, qoColor = tomQuest2.updateQuestObjectiveStatus(qid, oid)
				tomQuest2.questsObjectiveTextCells.questsTracker[qid][oid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTracker.colorizeQuestObjective and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor, tomQuest2.questsObjectives[qid][oid].desc)
				tomQuest2.questsObjectiveStatusCells.questsTracker[qid][oid].fs:SetFormattedText(qoStatus, tomQuest2.db.profile.questsTracker.colorizeQuestObjectiveCounter and qoColor or tomQuest2.db.profile.questsTracker.defaultQuestObjectiveColor)
			end
		end
	elseif info[#info] == "defaultQuestDescriptionColor" then
		for qid, qData in pairs(tomQuest2.quests) do
			if qData.questsTrackerDescription then
				qData.questsTrackerDescription.fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.questsTracker.defaultQuestDescriptionColor, qData.objective)
			end
		end
	end
	tomQuest2.setQuestsTrackerLayout()
end
local function questsTrackerQuestIconSizeSetter(info, value)
	tomQuest2.db.profile.questsTracker[info[#info]] = value
	for _, button in pairs(tomQuest2.specialItemButtons.questsTracker) do
		button:SetScale(value/button.height)
	end
	for _, button in pairs(tomQuest2.fakeItemButtons.questsTracker) do
		button:SetScale(value/button.height)
	end
	tomQuest2.setQuestsTrackerLayout()
end
local questsTrackerOptions = {type = "group", childGroups = "tab", name = LCo["Quests tracker"], get = questsTrackerOptGetter, set = questsTrackerOptSetter,
	args = {
		autotrack = {type = "group", name = LCo["Auto track"], set = optSetter, get = optGetter,
			args = {
				onLogIn = {name = LCo["Track all quests on log in"], type = "toggle", width = "full", order = 1},
				onQuestAccepted = {name = LCo["Auto track quests when you accept them"], type = "select", values = questsAutoTrack, width = "full", order = 2},
				onObjectiveProgress = {name = LCo["Auto track quests on objective progress"], type = "toggle", width = "full", order = 3},
				unTrackInactive = {name = LCo["Auto untrack inactive quests that were tracked on objective progress"], type = "toggle", width = "full", order = 4},
				unTrackDelay = {name = LCo["Auto untrack delay (in minutes)"], type = "range", min = 1, max = 30, step = 0.5, width = "full", order = 5},
				onSubZone = {name = LCo["Auto track quests on sub zone change"], type = "toggle", width = "full", order = 6},
				onZone = {name = LCo["Auto track quests on zone change"], type = "toggle", width = "full", order = 7},
				onDaily = {name = LCo["Auto track daily quests"], type = "toggle", width = "full", order = 8},
				onClass = {name = LCo["Auto track class quests"], type = "toggle", width = "full", order = 9},
			},
		},
		display = {type = "group", name = LCo["Display"],
			args = {
				overrideAnchor = {name = LCo["Override anchor and grow direction"], type = "toggle", set = questsTrackerOverrideAnchorSetter, width = "full", order = 1},
				overridePoint = {name = LCo["Overrided anchor point"], type = "select", values = anchorPointLOV, set = questsTrackerOverrideAnchorSetter, width = "full", order = 2},
				showByCategory = {name = LCo["Display quests by zone"], type = "toggle", set = byCategorySetter, width = "full", order = 3},
				autoCollapseCategory = {name = LCo["Automatically expand /collapse zone header on zone change"], type = "toggle", width = "full", order = 4},
				lockTracker = {name = LCo["Lock - unlock quests tracker"], type = "toggle", set = lockQuestsTrackerSetter, width = "full", order = 5},
				toggleTracker = {name = LCo["Display tracker"], type = "toggle", width = "full", order = 6},
				hideTrackerInCombat = {name = LCo["Hide quests tracker in combat"], type = "toggle", width = "full", order = 7},
				hideCompletedQuest = {name = LCo["Hide completed quest"], type = "select", values = hideQuests, width = "full", order = 8},
				hideCompletedObjective = {name = LCo["Hide completed quests objectives"], type = "select", values = hideObjectives, width = "full", order = 9},
				onlyCurrentZone = {name = LCo["Hide quests that are not in the current zone"], type = "toggle", width = "full", order = 10},
				limitQuestLevel = {name = LCo["Hide high level quests (You'll have to choose the maximum level difference)"], type = "toggle", width = "full", order = 11},
				maxLevelDiff = {name = LCo["Maximum level difference between the weighted quest level and your level"], type = "range", min = 0, max = 10, step = 1, width = "full", order = 12},
				hideZoneLevelRange = {name = LCo["Hide zone level range"], type = "toggle", set = hZLOptSetter, width = "full", order = 13},
				hideQuestTag = {name = LCo["Hide quest tag"], type = "toggle", set = hQTOptSetter, width = "full", order = 14},
			},
		},
		fonts = {type = "group", name = LCo["Tracker fonts"],
			args = {
				category = {type = "group", name = LCo["Zone"], set = questsTrackerCategoryFontSetter, inline = true, order = 1,
					args = {
						categoryFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						categoryFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						categoryFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				quests = {type = "group", name = LCo["Quests"], set = questsTrackerQuestFontSetter, inline = true, order = 2,
					args = {
						questFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						questFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						questFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				questsObjectives = {type = "group", name = LCo["Quests objectives"], set = questsTrackerQuestObjectiveFontSetter, inline = true, order = 3,
					args = {
						questObjectiveFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						questObjectiveFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						questObjectiveFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
			},
		},
		appearance = {type = "group", name = LCo["Tracker appearance"],
			args = {
				backdrop = {type = "group", name = LCo["Backdrop"], inline = true, order = 1,
					args = {
						bgTexture = {name = LCo["Background texture"], type = "select", dialogControl = "LSM30_Background", values = AceGUIWidgetLSMlists.background, set = questsTrackerBackdropSetter, order = 1},
						borderTexture = {name = LCo["Border texture"], type = "select", dialogControl = "LSM30_Border", values = AceGUIWidgetLSMlists.border, set = questsTrackerBackdropSetter, order = 2},
						borderWidth = {name = LCo["Border width"], type = "range", min = 1, max = 24, step = 1, set = questsTrackerBackdropSetter, order = 3},
						borderInset = {name = LCo["Inset"], type = "range", min = 0, max = 10, step = 1, set = questsTrackerBackdropSetter, order = 4},
						backDropColor = {name = LCo["Quests tracker background color"], type = "color", hasAlpha = true, get = questsTrackerColorGetter, set = questsTrackerColorSetter, width = "full", order = 5},
						borderColor = {name = LCo["Quests tracker border color"], type = "color", hasAlpha = true, get = questsTrackerColorGetter, set = questsTrackerColorSetter, width = "full", order = 6},
					},
				},
				scale = {name = LCo["Quests tracker scale"], type = "range", min = 0, max = 1, step = 0.1, width = "full", order = 7},
				hasMaxHeight = {name = LCo["Set quests tracker maximum height after which it will be scrollable"], type = "toggle", width = "full", order = 8},
				maxHeight = {name = LCo["Quests tracker max height"], type = "range", min = 100, max = 2000, step = 5, width = "full", order = 9},
				showCollapseButton = {name = LCo["Show collapse button"], type = "toggle", width = "full", order = 10},
				collapseButtonSize = {name = LCo["Collapse button size"], type = "range", min = 10, max = 40, step = 1, set = questsTrackerCollapseButtonSizeSetter, width = "full", order = 11},
				questsCellPadding = {name = LCo["Quests cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 12},
				objectivesCellPadding = {name = LCo["Quests objectives cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 13},
				maxWidth = {name = LCo["Quests tracker column max width"], type = "range", min = 50, max = 600, step = 5, width = "full", order = 14},
				minWidth = {name = LCo["Quests tracker column min width"], type = "range", min = 50, max = 600, step = 5, width = "full", order = 15},
				statusMaxWidth = {name = LCo["Quests tracker status column max width"], type = "range", min = 30, max = 200, step = 5, width = "full", order = 16},
				statusMinWidth = {name = LCo["Quests tracker status column min width"], type = "range", min = 30, max = 200, step = 5, width = "full", order = 17},
			},
		},
		color = {type = "group", name = LCo["Colors"], set = setquestsTrackerColors,
			args = {
				colorizeCategoryName = {name = LCo["Color quests header name"], type = "toggle", width = "full", order = 3},
				defaultCategoryNameColor = {name = LCo["Default header name color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 4},
				colorizeCategoryLevel = {name = LCo["Color quests header level"], type = "toggle", width = "full", order = 5},
				defaultCategoryLevelColor = { name = LCo["Default header level color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 6},
				colorizeQuestName = {name = LCo["Color quests name"], type = "toggle", width = "full", order = 7},
				defaultQuestNameColor = {name = LCo["Default quest name color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 8},
				colorizeQuestLevel = {name = LCo["Color quests level"], type = "toggle", width = "full", order = 9},
				defaultQuestLevelColor = {name = LCo["Default quest level color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 10},
				colorizeQuestTimer = {name = LCo["Color quests timer"], type = "toggle", width = "full", order = 11},
				defaultQuestTimerColor = {name = LCo["Default quest timer color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 12},
				colorizeQuestObjective = {name = LCo["Color quests objectives"], type = "toggle", width = "full", order = 13},
				defaultQuestObjectiveColor = {name = LCo["Default quest objective color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 14},
				colorizeQuestObjectiveCounter = { name = LCo["Color quests objectives counter"], type = "toggle", width = "full", order = 15},
				defaultQuestObjectiveCounterColor = {name = LCo["Default quest objective counter color"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 16},
				defaultQuestDescriptionColor = {name = LCo["Default quest description color for quest without objective"], type = "color", hasAlpha = false, get = questsTrackerHexColorGetter, set = questsTrackerHexColorSetter, width = "full", order = 17},
			},
		},
		questsItems = {type = "group", name = LCo["Quests items"],
			args = {
				showQuestItemIcon = {name = LCo["Show special quest item icon behind quest name"], type = "toggle", width = "full", order = 1},
				questItemIconSize = {name = LCo["Quest item icon size"], type = "range", min = 6, max = 36, step = 1, width = "full", order = 2},
				oorColor = {name = LCo["Out of range color"], type = "color", hasAlpha = false, get = questsTrackerColorGetter, set = questsTrackerColorSetter,width = "full", order = 3},
				showItemsStartingQuest = {name = LCo["Show items starting a quest in the tracker"], type = "toggle", width = "full", order = 4},
				showUsableQuestItemsNotAttached = {name = LCo["Show usable quest item not linked to a quest"], type = "toggle", width = "full", order = 5},
				otherItemIconSize = {name = LCo["Other quest item icon size"], type = "range", min = 6, max = 36, step = 1, width = "full", order = 6},
			},
		},
		trackerInteraction = {type = "group", name = LCo["Tracker interaction"],
			args = {
				disableInteraction = {type = "toggle", name = LCo["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make it scrollable."], width = "full", order = 1},
				overTooltip = {name = LCo["Display quest tooltip on mouse over"], type = "toggle", width = "full", hidden = hideQuestsTrackerInteraction, order = 2},
				mouseRightShift = {name = LCo["Action on right shift click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 3},
				mouseRightControl = {name = LCo["Action on right control click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 4},
				mouseRightAlt = {name = LCo["Action on right alt click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 5},
				mouseRight = {name = LCo["Action on right click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 6},
				mouseLeftShift = {name = LCo["Action on left shift click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 7},
				mouseLeftControl = {name = LCo["Action on left control click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 8},
				mouseLeftAlt = {name = LCo["Action on left alt click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 9},
				mouseLeft = {name = LCo["Action on left click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 10},
				mouseMiddleShift = {name = LCo["Action on middle shift click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 11},
				mouseMiddleControl = {name = LCo["Action on middle control click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 12},
				mouseMiddleAlt = {name = LCo["Action on middle alt click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 13},
				mouseMiddle = {name = LCo["Action on middle click"], type = "select", values = tomQuest2.interactivityList, width = "full", hidden = hideQuestsTrackerInteraction, order = 14},
			},
		},
	},
}

-----------------------------------------
--- Achievements Tracker option table ---
-----------------------------------------
local function achievementsTrackerOverrideAnchorSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	setTrackerpoint(_G["tomQuest2AchievementsTracker"])
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementCatFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	_G["cATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.categoryFont), tomQuest2.db.profile.achievementsTracker.categoryFontSize, tomQuest2.db.profile.achievementsTracker.categoryFontOutline)
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementSubCatFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	_G["scATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.subCategoryFont), tomQuest2.db.profile.achievementsTracker.subCategoryFontSize, tomQuest2.db.profile.achievementsTracker.subCategoryFontOutline)
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	_G["aATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementFont), tomQuest2.db.profile.achievementsTracker.achievementFontSize, tomQuest2.db.profile.achievementsTracker.achievementFontOutline)
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementCritFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	_G["acATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.achievementCriterionFont), tomQuest2.db.profile.achievementsTracker.achievementCriterionFontSize, tomQuest2.db.profile.achievementsTracker.achievementCriterionFontOutline)
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementStatusBarFontSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	_G["sbATrFontTQ2"]:SetFont(LSM:Fetch("font", tomQuest2.db.profile.achievementsTracker.statusBarFont), tomQuest2.db.profile.achievementsTracker.statusBarFontSize, tomQuest2.db.profile.achievementsTracker.statusBarFontOutline)
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementOptGetter(info)
	return tomQuest2.db.profile.achievementsTracker[info[#info]]
end
local function achievementOptSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.setAchievementsTrackerLayout()
end
local function timedAchievementSetter(info, value)
	tomQuest2.db.profile[info[#info]] = value
	if not tomQuest2.moduleState.achievementsTracker then return end
	if value then
		tomQuest2:RegisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	else
		tomQuest2:UnregisterEvent("TRACKED_ACHIEVEMENT_UPDATE")
	end
end
local function achievementInQuestsTrackerSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	if value then
		tomQuest2.ACHIEVEMENTS_TRACKER_FRAME = "questsTracker"
		for _, frame in pairs(tomQuest2.achievementCategoryTextCells) do
			frame:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
		end
		for _, frame in pairs(tomQuest2.achievementCategoryStatusCells) do
			frame:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
		end
		for _, aData in pairs(tomQuest2.achievements) do
			aData.cell:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
			if aData.DescriptionCell then
				aData.DescriptionCell:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
			end
			if aData.statusBar then
				aData.statusBar:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
			end
			if aData.criteria then
				for _, cData in pairs(aData.criteria) do
					if cData.textCell then
						cData.textCell:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
						cData.statusCell:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
					elseif cData.booleanCell then
						cData.booleanCell:SetParent(_G["tomQuest2QuestsTrackerScrollChild"])
					end
				end
			end
		end
		tomQuest2.setAchievementsTrackerLayout()
	else
		tomQuest2.ACHIEVEMENTS_TRACKER_FRAME = "achievementsTracker"
		for _, frame in pairs(tomQuest2.achievementCategoryTextCells) do
			frame:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
		end
		for _, frame in pairs(tomQuest2.achievementCategoryStatusCells) do
			frame:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
		end
		for _, aData in pairs(tomQuest2.achievements) do
			aData.cell:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
			if aData.DescriptionCell then
				aData.DescriptionCell:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
			end
			if aData.statusBar then
				aData.statusBar:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
			end
			if aData.criteria then
				for _, cData in pairs(aData.criteria) do
					if cData.textCell then
						cData.textCell:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
						cData.statusCell:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
					elseif cData.booleanCell then
						cData.booleanCell:SetParent(_G["tomQuest2AchievementsTrackerScrollChild"])
					end
				end
			end
		end
		tomQuest2.setQuestsTrackerLayout()
		tomQuest2.setAchievementsTrackerLayout()
	end
end
local function lockAchievementTrackerSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	_G["tomQuest2AchievementsTracker"]:SetMovable(not value)
end
local function byCategorySetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	if not value then
		for _, frame in pairs(tomQuest2.achievementCategoryTextCells) do
			frame:Hide()
		end
		for _, frame in pairs(tomQuest2.achievementCategoryStatusCells) do
			frame:Hide()
		end
	end
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementsTrackerBackdropSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	tomQuest2.setBackDrop("achievementsTracker")
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementsTrackerCategoryCollapseButtonSizeSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	for cid in pairs(tomQuest2.achievementsCategories) do
		tomQuest2.categoryButtons.achievementsTracker[cid]:SetScale(value/tomQuest2.categoryButtons.achievementsTracker[cid].height)
	end
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementsTrackerSubCategoryCollapseButtonSizeSetter(info, value)
	for cid, cData in pairs(tomQuest2.achievementsCategories) do
		if next(cData.subCategory) then
			for scid, scData in pairs(cData.subCategory) do
				tomQuest2.categoryButtons.achievementsTracker[scid]:SetScale(value/tomQuest2.categoryButtons.achievementsTracker[scid].height)
			end
		end
	end
	tomQuest2.setAchievementsTrackerLayout()
end
local function achievementsStatusBarTextureSetter(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	if next(tomQuest2.achievementStatusBars) then
		for _, statusBar in pairs(tomQuest2.achievementStatusBars) do
			statusBar:SetStatusBarTexture(LSM:Fetch("statusbar", value))
			statusBar.bg:SetTexture(LSM:Fetch("statusbar", value))
		end
	end
end
local function achievementColorGetter(info)
	return unpack(tomQuest2.db.profile.achievementsTracker[info[#info]])
end
local function achievementColorSetter(info, r, g, b, a)
	tomQuest2.db.profile.achievementsTracker[info[#info]][1] = r
	tomQuest2.db.profile.achievementsTracker[info[#info]][2] = g
	tomQuest2.db.profile.achievementsTracker[info[#info]][3] = b
	tomQuest2.db.profile.achievementsTracker[info[#info]][4] = a
	if info[#info] == "defaultStatusBarColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementStatusBars[aid] then
				local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, 1)
				local color = getColorFromCompletionHEX(quantity / reqQuantity)
				tomQuest2.updateStatusbar(aid, cType, quantity, reqQuantity, color)
			end
		end
	end
	tomQuest2.setAchievementsTrackerLayout()
end
local function setAchievementTrackerColors(info, value)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = value
	if info[#info] == "colorizeACName" or info[#info] == "colorizeACCounter" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			local numTA, numTCA = GetCategoryNumAchievements(cid)
			local name = GetCategoryInfo(cid)
			local color = getColorFromCompletionHEX(numTCA / numTA)
			tomQuest2.updateAchievementCategoryCells(cid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
		end
	elseif info[#info] == "colorizeASCName" or info[#info] == "colorizeASCCounter" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			if next(cData.subCategory) then
				for scid, scData in pairs(cData.subCategory) do
					local numTA, numTCA = GetCategoryNumAchievements(cid)
					local name = GetCategoryInfo(scid)
					local color = getColorFromCompletionHEX(numTCA / numTA)
					tomQuest2.updateAchievementCategoryCells(scid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
				end
			end
		end
	elseif "colorizeACrName" or info[#info] == "colorizeACrStatus" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if aData.criteria then
				for cid in pairs(aData.criteria) do
					local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, cid)
					local color = getColorFromCompletionHEX(quantity / reqQuantity)
					tomQuest2.achievementCriterionTextCells[aid][cid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name)
					tomQuest2.achievementCriterionStatusCells[aid][cid].fs:SetFormattedText("|cff%s / %s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrStatus and color or tomQuest2.db.profile.achievementsTracker.defaultACrStatusColor, quantity, reqQuantity)
				end
			end
		end
	elseif info[#info] == "colorizeStatusBar" or info[#info] == "colorizeStatusBarText" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementStatusBars[aid] then
				local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, 1)
				local color = getColorFromCompletionHEX(quantity / reqQuantity)
				tomQuest2.updateStatusbar(aid, cType, quantity, reqQuantity, color)
			end
		end
	end
end
local function achievementHexColorGetter(info)
	return tonumber(sub(tomQuest2.db.profile.achievementsTracker[info[#info]], 1, 2), 16) / 255, tonumber(sub(tomQuest2.db.profile.achievementsTracker[info[#info]], 3, 4), 16) / 255, tonumber(sub(tomQuest2.db.profile.achievementsTracker[info[#info]], 5, 6), 16) / 255
end
local function achievementHexColorSetter(info,r, g, b)
	tomQuest2.db.profile.achievementsTracker[info[#info]] = format("%02x%02x%02x", r*255, g*255, b*255)
	if info[#info] == "defaultACNameColor" or info[#info] == "defaultACCounterColor" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			local numTA, numTCA = GetCategoryNumAchievements(cid)
			local name = GetCategoryInfo(cid)
			local color = getColorFromCompletionHEX(numTCA / numTA)
			tomQuest2.updateAchievementCategoryCells(cid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
		end
	elseif info[#info] == "defaultASCNameColor" or info[#info] == "defaultASCCounterColor" then
		for cid, cData in pairs(tomQuest2.achievementsCategories) do
			if next(cData.subCategory) then
				for scid, scData in pairs(cData.subCategory) do
					local numTA, numTCA = GetCategoryNumAchievements(cid)
					local name = GetCategoryInfo(scid)
					local color = getColorFromCompletionHEX(numTCA / numTA)
					tomQuest2.updateAchievementCategoryCells(scid, tomQuest2.db.profile.achievementsTracker.colorizeACName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name, tomQuest2.db.profile.achievementsTracker.colorizeACCounter and color or tomQuest2.db.profile.achievementsTracker.defaultACCounterColor, numTCA, numTA)
				end
			end
		end
	elseif info[#info] == "defaultANameColor" then
		for aid in pairs(tomQuest2.achievements) do
			tomQuest2.achievementCells[aid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.defaultANameColor, tomQuest2.achievements[aid].name)
		end
	elseif info[#info] == "defaultACrNameColor" or info[#info] == "defaultACrStatusColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if aData.criteria then
				for cid in pairs(aData.criteria) do
					local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, cid)
					local color = getColorFromCompletionHEX(quantity / reqQuantity)
					tomQuest2.achievementCriterionTextCells[aid][cid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrName and color or tomQuest2.db.profile.achievementsTracker.defaultACrNameColor, name)
					tomQuest2.achievementCriterionStatusCells[aid][cid].fs:SetFormattedText("|cff%s / %s|r", tomQuest2.db.profile.achievementsTracker.colorizeACrStatus and color or tomQuest2.db.profile.achievementsTracker.defaultACrStatusColor, quantity, reqQuantity)
				end
			end
		end
	elseif info[#info] == "defaultStatusBarTextColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementStatusBars[aid] then
				local name, cType, completed, quantity, reqQuantity, _, flags = GetAchievementCriteriaInfo(aid, 1)
				local color = getColorFromCompletionHEX(quantity / reqQuantity)
				tomQuest2.updateStatusbar(aid, cType, quantity, reqQuantity, color)
			end
		end
	elseif info[#info] == "defaultDescriptionColor" then
		for aid, aData in pairs(tomQuest2.achievements) do
			if tomQuest2.achievementDescriptionCells[aid] then
				tomQuest2.achievementDescriptionCells[aid].fs:SetFormattedText("|cff%s%s|r", tomQuest2.db.profile.achievementsTracker.defaultDescriptionColor, tomQuest2.achievements[aid].description)
			end
		end
	end
	tomQuest2.setAchievementsTrackerLayout()
end
local achievementsTrackerOptions = {type = "group", childGroups = "tab", name = LCo["Achievements tracker"], get = achievementOptGetter, set = achievementOptSetter,
	args = {
		autotrack = {type = "group", name = LCo["Auto track"],
			args = {timedAchievements = {name = LCo["Timed achievements"], type = "toggle", get = optGetter, set = timedAchievementSetter, width = "full", order = 1}},
		},
		display = {type = "group", name = LCo["Display"],
			args = {
				overrideAnchor = {name = LCo["Override anchor and grow direction"], type = "toggle", set = achievementsTrackerOverrideAnchorSetter, width = "full", order = 1},
				overridePoint = {name = LCo["Overrided anchor point"], type = "select", values = anchorPointLOV, set = achievementsTrackerOverrideAnchorSetter, width = "full", order = 2},
				showInQuestTracker = {name = LCo["Show achievements in quests tracker"], type = "toggle", set = achievementInQuestsTrackerSetter, width = "full", order = 3},
				positionInQuestTracker = {name = LCo["Position of achievements relative to quests in the tracker"], type = "select", values = achievementPosition, set = achievementInQuestsTrackerSetter, width = "full", order = 4},
				lockTracker = {name = LCo["Lock - unlock achievements tracker"], type = "toggle", set = lockAchievementTrackerSetter, width = "full", order = 5},
				toggleTracker = {name = LCo["Display tracker"], type = "toggle", width = "full", order = 6},
				hideTrackerInCombat = {name = LCo["Hide achievements tracker in combat"], type = "toggle", width = "full", order = 7},
				hideCompletedCriteria = {name = LCo["Hide completed achievement criteria"], type = "toggle", width = "full", order = 8},
				showByCategory = {name = LCo["Display achievements by category and subcategory"], type = "toggle", set = byCategorySetter, width = "full", order = 9},
			},
		},
		fonts = {type = "group", name = LCo["Tracker fonts"],
			args = {
				category = {type = "group", name = LCo["Achievement category"], set = achievementCatFontSetter, inline = true, order = 1,
					args = {
						categoryFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						categoryFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						categoryFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				subCategory = {type = "group", name = LCo["Achievement sub category"], set = achievementSubCatFontSetter, inline = true, order = 2,
					args = {
						subCategoryFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						subCategoryFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						subCategoryFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				achievement = {type = "group", name = LCo["Achievement"], set = achievementFontSetter, inline = true, order = 3,
					args = {
						achievementFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						achievementFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						achievementFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				achievementCriteria = {type = "group", name = LCo["Achievement criteria"], set = achievementCritFontSetter, inline = true, order = 4,
					args = {
						achievementCriterionFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						achievementCriterionFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						achievementCriterionFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
				statusBar = {type = "group", name = LCo["Status bar"], set = achievementStatusBarFontSetter, inline = true, order = 5,
					args = {
						statusBarFont = {name = LCo["Font"], type = "select", dialogControl = "LSM30_Font", values = AceGUIWidgetLSMlists.font, width = "full", order = 1},
						statusBarFontSize = {name = LCo["Font size"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 2},
						statusBarFontOutline = {name = LCo["Font outline"], type = "select", values = outlines, width = "full", order = 3},
					},
				},
			},
		},
		appearance = {type = "group", name = LCo["Tracker appearance"],
			args = {
				backdrop = {type = "group", name = LCo["Backdrop"], inline = true, order = 1,
					args = {
						bgTexture = {name = LCo["Background texture"], type = "select", dialogControl = "LSM30_Background", values = AceGUIWidgetLSMlists.background, set = achievementsTrackerBackdropSetter, order = 1},
						borderTexture = {name = LCo["Border texture"], type = "select", dialogControl = "LSM30_Border", values = AceGUIWidgetLSMlists.border, set = achievementsTrackerBackdropSetter, order = 2},
						borderWidth = {name = LCo["Border width"], type = "range", min = 1, max = 24, step = 1, set = achievementsTrackerBackdropSetter, order = 3},
						borderInset = {name = LCo["Inset"], type = "range", min = 0, max = 10, step = 1, set = achievementsTrackerBackdropSetter, order = 4},
						backDropColor = {name = LCo["Achievements tracker background color"], type = "color", hasAlpha = true, get = achievementColorGetter, set = achievementColorSetter, width = "full", order = 5},
						borderColor = {name = LCo["Achievements tracker border color"], type = "color", hasAlpha = true, get = achievementColorGetter, set = achievementColorSetter, width = "full", order = 6},
					},
				},
				scale = {name = LCo["Achievements tracker scale"], type = "range", min = 0, max = 1, step = 0.1, width = "full", order = 5},
				hasMaxHeight = {name = LCo["Set achievements tracker maximum height after which it will be scrollable"], type = "toggle", width = "full", order = 6},
				maxHeight = {name = LCo["Achievements tracker max height"], type = "range", min = 100, max = 2000, step = 5, width = "full", order = 7},
				showCollapseButton = {name = LCo["Show collapse button"], type = "toggle", width = "full", order = 8},
				categoryCollapseButtonSize = {name = LCo["Category collapse button size"], type = "range", min = 10, max = 40, step = 1, set = achievementsTrackerCategoryCollapseButtonSizeSetter, width = "full", order = 9},
				subCategoryCollapseButtonSize = {name = LCo["Sub category collapse button size"], type = "range", min = 10, max = 40, step = 1, set = achievementsTrackerSubCategoryCollapseButtonSizeSetter, width = "full", order = 10},
				showAchievementIcon = {name = LCo["Display achievements icon"], type = "toggle", width = "full", order = 11},
				achievementIconSize = {name = LCo["Achievement image size"], type = "range", min = 10, max = 40, step = 1, width = "full", order = 12},
				subCategoryCellPadding = {name = LCo["Sub category cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 13},
				achievementCellPadding = {name = LCo["Achievement cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 14},
				criteriaCellPadding = {name = LCo["Achievement criteria cell padding"], type = "range", min = 0, max = 50, step = 1, width = "full", order = 15},
				maxWidth = {name = LCo["Achievements tracker column max width"], type = "range", min = 50, max = 600, step = 5, width = "full", order = 17},
				minWidth = {name = LCo["Achievements tracker column min width"], type = "range", min = 50, max = 600, step = 5, width = "full", order = 18},
				statusMaxWidth = {name = LCo["Achievements tracker status column max width"], type = "range", min = 30, max = 200, step = 5, width = "full", order = 19},
				statusMinWidth = {name = LCo["Achievements tracker status column min width"], type = "range", min = 30, max = 200, step = 5, width = "full", order = 20},
				numBooleanCriteriaColumns = {name = LCo["Number of columns in the achievements tracker for boolean criteria"], type = "range", min = 1, max = 8, step = 1, width = "full", order = 21},
			},
		},
		color = {type = "group", name = LCo["Colors"],
			args = {
				colorizeACName = {name = LCo["Color achievements category name"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 3},
				defaultACNameColor = {name = LCo["Default achievements category name color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 4},
				colorizeACCounter = {name = LCo["Color achievements category counter"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 5},
				defaultACCounterColor = {name = LCo["Default achievements category counter color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 6},
				colorizeASCName = {name = LCo["Color achievements subcategory name"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 7},
				defaultASCNameColor = {name = LCo["Default achivements subcategory name color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 8},
				colorizeASCCounter = {name = LCo["Color achievements subcategory counter"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 9},
				defaultASCCounterColor = {name = LCo["Default achievements subcategory counter color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 10 },
				defaultANameColor = {name = LCo["Default achievements name color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 12},
				colorizeATimer = {name = LCo["Color achievement timer"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 13},
				defaultATimerColor = {name = LCo["Default achievement timer color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 14},
				colorizeACrName = {name = LCo["Color achievements criteria name"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 15},
				defaultACrNameColor = {name = LCo["Default achievements criteria name color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 16},
				colorizeACrStatus = {name = LCo["Color achievements cirteria status"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 17},
				defaultACrStatusColor = {name = LCo["Default achievements criteria status color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 18},
				colorizeStatusBar = {name = LCo["Color status bar"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 19},
				defaultStatusBarColor = {name = LCo["Default status bar color"], type = "color", hasAlpha = false, get = achievementColorGetter, set = achievementColorSetter, width = "full", order = 20},
				colorizeStatusBarText ={name = LCo["Color status bar text"], type = "toggle", set = setAchievementTrackerColors, width = "full", order = 21},
				defaultStatusBarTextColor = {name = LCo["Default status bar text color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 22},
				defaultDescriptionColor = {name = LCo["Default achievement description color"], type = "color", hasAlpha = false, get = achievementHexColorGetter, set = achievementHexColorSetter, width = "full", order = 23},
			},
		},
		statusBar = {type = "group", name = LCo["Status bar"],
			args = {
				displayAsStatusBar = {name = LCo["Display status bar for criteria progression when possible"], type = "toggle", width = "full", order = 1},
				statusBarTexture = {name = LCo["Status bar texture"], type = "select", dialogControl = "LSM30_Statusbar", values = AceGUIWidgetLSMlists.statusbar, set = achievementsStatusBarTextureSetter, order = 2},
				statusBarHeight = {name = LCo["Status bar height"], type = "range", min = 6, max = 20, step = 1, width = "full", order = 3},
				leftStatusBarPadding = {name = LCo["Left padding of the status bar"], type = "range", min = 0, max = 100, step = 1, width = "full", order = 4},
				rightStatusBarPadding = {name = LCo["Right padding of the status bar"], type = "range", min = 0, max = 100, step = 1, width = "full", order = 5},
			},
		},
		interaction = {type = "group", name = LCo["Tracker interaction"],
			args = {
				disableInteraction = {name = LCo["Completely disable tracker interaction, should not be used when display by category is used or if you wish to make it scrollable."], type = "toggle", width = "full", order = 1},
				overTooltip = {name = LCo["Display achievement tooltip on mouse over"], type = "toggle", width = "full", hidden = hideAchievementInteraction, order = 2},
				mouseRightShift = {name = LCo["Action on right shift click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 3},
				mouseRightControl = {name = LCo["Action on right control click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 4},
				mouseRightAlt = {name = LCo["Action on right alt click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 5},
				mouseRight = {name = LCo["Action on right click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 6},
				mouseLeftShift = {name = LCo["Action on left shift click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 7},
				mouseLeftControl = {name = LCo["Action on left control click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 8},
				mouseLeftAlt = {name = LCo["Action on left alt click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 9},
				mouseLeft = {name = LCo["Action on left click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 10},
				mouseMiddleShift = {name = LCo["Action on middle shift click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 11},
				mouseMiddleControl = {name = LCo["Action on middle control click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 12},
				mouseMiddleAlt = {name = LCo["Action on middle alt click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 13},
				mouseMiddle = {name = LCo["Action on middle click"], type = "select", values = achievementInteractivityList, width = "full", hidden = hideAchievementInteraction, order = 14},
			},
		},
	},
}

-------------------------
--- Make option panel ---
-------------------------
local profilesOptions = LibStub("AceDBOptions-3.0"):GetOptionsTable(tomQuest2.db)
ACR:RegisterOptionsTable("tomQuest2", options)
tomQuest2.mainOptionFrame = ACD:AddToBlizOptions("tomQuest2", "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_profiles", profilesOptions)
tomQuest2.profilesOptionFrame = ACD:AddToBlizOptions("tomQuest2_profiles", LCo["Profiles"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_achievementsTracker", achievementsTrackerOptions)
ACD:AddToBlizOptions("tomQuest2_achievementsTracker", LCo["Achievements tracker"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_announce", announceOptions)
ACD:AddToBlizOptions("tomQuest2_announce", LCo["Announce"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_dailyTooltip", dailyTooltipOptions)
ACD:AddToBlizOptions("tomQuest2_dailyTooltip", LCo["Daily tooltip"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_notifications", notificationsOptions)
ACD:AddToBlizOptions("tomQuest2_notifications", LCo["Notifications"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsGivers", questsGiversOptions)
ACD:AddToBlizOptions("tomQuest2_questsGivers", LCo["Quests givers"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsInformations", questsInformationsOptions)
ACD:AddToBlizOptions("tomQuest2_questsInformations", LCo["Information"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsTooltip", questsTooltipOptions)
ACD:AddToBlizOptions("tomQuest2_questsTooltip", LCo["Quests tooltip"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_questsTracker", questsTrackerOptions)
ACD:AddToBlizOptions("tomQuest2_questsTracker", LCo["Quests tracker"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_soundNotifications", soundNotificationsOptions)
ACD:AddToBlizOptions("tomQuest2_soundNotifications", LCo["Sound notifications"], "tomQuest2")
ACR:RegisterOptionsTable("tomQuest2_tooltips", tooltipsOptions)
ACD:AddToBlizOptions("tomQuest2_tooltips", LCo["Tooltips"], "tomQuest2")